/*
 * Basic subrutines
 */


#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#ifndef INADDR_NONE
#define INADDR_NONE 0xffffffff
#endif

extern int errno;
extern char *syserrlist[];
u_long inet_addr();

/* Alocate and connect socket using TCP/IP or UDP protocol */
int connectsock(char* host, char* service, char* protocol)
{

    /* Description of data base entry for a single host.
     struct hostent
     {
     char *h_name;		 Official name of host.
     char **h_aliases;		 Alias list.
     int h_addrtype;		 Host address type.
     int h_length;			 Length of address.
     char **h_addr_list;		 List of addresses from name server.
     #define	h_addr	h_addr_list[0]	 Address, for backward compatibility.
     };*/
    struct hostent* phe;        /* host inf. entry */

    /* Description of data base entry for a single service.
     struct servent
     {
     char *s_name;		 Official service name.
     char **s_aliases;		 Alias list.
     int s_port;		 Port number.
     char *s_proto;		 Protocol to use.
     };*/
    struct servent* pse;        /* service inf. entry */

    /* Description of data base entry for a single service.
    struct protoent
    {
        char *p_name;			 Official protocol name.
        char **p_aliases;		 Alias list.
        int p_proto;			 Protocol number.
    };*/
    struct protoent* ppe;       /* protocol inf. entry */

    struct sockaddr_in sin;
    int s, type;
    char buf[256];

    /* sets structure sin to zero */
    bzero((char*)&sin, sizeof(sin));
    sin.sin_family=AF_INET;

    /* Map service name to port number */
    if (pse=getservbyname(service, protocol))
    {
        sin.sin_port=pse->s_port;
    }
    else if ((sin.sin_port=htons((u_short)atoi(service))) == 0)
    {
        sprintf(buf, "Can't get \"%s\" service entry\n", service);
        perror(buf);
        exit(1);
    }

    /* Map hostname to IP address, allowing for dotted decimal */
    if (phe=gethostbyname(host))
    {
        bcopy(phe->h_addr, (char*)&sin.sin_addr, phe->h_length);
    }
    else if ((sin.sin_addr.s_addr=inet_addr(host)) == INADDR_NONE)
    {
        sprintf(buf, "Can't get \"%s\" host entry.\n", host);
        perror(buf);
        exit(1);
    }

    /* Map protocol name to protocol number */
    if ((ppe=getprotobyname(protocol)) == NULL)
    {
        sprintf(buf, "Can't get \"%s\" protocol entry.\n", protocol);
        perror(buf);
        exit(1);
    }

    /* Use protocol to chose a socket type */
    if (strcmp(protocol, "udp") == 0)
        type=SOCK_DGRAM;
    else
        type=SOCK_STREAM;

    /* Allocate the socket */
    s=socket(PF_INET, type, ppe->p_proto);
    if (s < 0)
    {
        sprintf(buf, "Can't create socket.\n");
        perror(buf);
        exit(1);
    }

    /* Connect the socket */
    if (connect(s, (struct sockaddr*)&sin, sizeof(sin)) < 0)
    {
        sprintf(buf, "Can't connect to %s.%d:\n", host, ppe->p_proto);
        perror(buf);
        exit(1);
    }

    return s;
}

int connectTCP (char* host, char* service)
{
    return connectsock(host, service, "tcp");
}

int connectUDP (char* host, char* service)
{
    return connectsock(host, service, "udp");
}

int listensock(char* service, int backlog)
{
    /*
     * The servent struct is defined in <netdb.h> as follows:
     *  struct hostent{
     *   char *h_name;
     *   char **h_aliases;
     *   int h_addrtype;
     *   int h_length;
     *   char **h_addr_list;
     *   #define h_addr	h_addr_list[0]
     *  };
     */
    struct hostent* phe;        /* host inf. entry */

    /*
     * The servent struct is defined in <netdb.h> as follows:
     *  struct servent{
     *   char *s_name;
     *   char **s_aliases;
     *   int s_port;
     *   cahr *s_proto;
     *  }
     */
    struct servent* pse;        /* service inf. entry */

    /*
     * The protoent struct is defined in <netdb.h> as follows:
     *  struct protoent{
     *   char *p_name;
     *   char **p_aliases;
     *   int p_proto;
     *  }
     */
    struct protoent* ppe;       /* protocol inf. entry */

    char *protocol="tcp";       /* listen is only for STREAM and SEQPACKET */
    const int type=SOCK_STREAM; /* sock. type */
    struct sockaddr_in sin;     /* server adress */
    int s;                      /* sock. descriptor */
    char servername[256];       /* server name */
    char buf[256];

    bzero((char*)&sin, sizeof(sin));
    sin.sin_family=AF_INET;

    /* Map service name to port number */
    if (pse=getservbyname(service, protocol))
    {
        sin.sin_port=pse->s_port;
    }
    else if ((sin.sin_port=htons((u_short)atoi(service))) == 0)
    {
        sprintf(buf, "Can't get \"%s\" service entry\n", service);
        perror(buf);
        exit(1);
    }

    /* Get and map servername to IP address, allowing for dotted decimal */
    if(gethostname(servername, 256) < 0)
    {
        perror("Can't get host name.");
        exit(1);
    }
    if (phe=gethostbyname(servername))
    {
        bcopy(phe->h_addr, (char*)&sin.sin_addr, phe->h_length);
    }
    else if ((sin.sin_addr.s_addr=inet_addr(servername)) == INADDR_NONE)
    {
        sprintf(buf, "Can't get \"%s\" host entry.\n", servername);
        perror(buf);
        exit(1);
    }

    /* Map protocol name to protocol number */
    if ((ppe=getprotobyname(protocol)) == NULL)
    {
        sprintf(buf, "Can't get \"%s\" protocol entry.\n", protocol);
        perror(buf);
        exit(1);
    }

    /* Allocate the socket */
    s=socket(PF_INET, type, ppe->p_proto);
    if (s < 0)
    {
        sprintf(buf, "Can't create socket.\n");
        perror(buf);
        exit(1);
    }

    /* Bind the socket */
    if (bind(s, (struct sockaddr*)&sin, sizeof(sin)) < 0)
    {
        sprintf(buf, "Can't bind to %s.%d:\n", servername, pse->s_port);
        perror(buf);
        exit(1);
    }
    /* Listen the socket */
    if (listen(s, backlog) < 0)
    {
        sprintf(buf, "Can't listen to %s.%d:\n", servername, ppe->p_proto);
        perror(buf);
        exit(1);
    }
    return s;
}
